"""
MailBackend
~~~~~~~~~~~~~
reads smtp config from ini file and sends out emails as directed by main

    :copyright: 2015 by Matthias Kauer
    :license: BSD
"""

from smtplib import SMTP_SSL, SMTP
import os
import ConfigParser

class MailBackend(object):
    def __init__(self, scriptpath):
        self.cfg = ConfigParser.SafeConfigParser()
        fname = os.path.realpath(scriptpath)

        try:
            self.cfg.read(fname)
            self.server = self.cfg.get('Mail', 'smtpserver')
            self.port = self.cfg.get('Mail', 'smtpport')
            self.mailuser = self.cfg.get('Mail','user')
            self.mailpass = self.cfg.get('Mail','pass')
            self.fromaddr = self.cfg.get('Mail', 'fromaddr')
            self.security = self.cfg.get('Mail', 'security')
            self.loggedin = False
        except Exception, e:
            import sys
            sys.exit( "%s appears not to be an .ini file with the appropriate sections.\n \
Call 'croncoat --ini' for an example layout of the .ini file" %scriptpath )


    def sendmail(self, emailMsg):
        if(not self.loggedin):
            if self.security.lower() == 'starttls':
                self.smtp = SMTP(self.server, int(self.port))
                self.smtp.starttls()
            elif self.security.lower() == 'none':
                self.smtp = SMTP(self.server, int(self.port))
            elif self.security.lower() == 'ssl':
                self.smtp = SMTP_SSL(self.server, self.port)

            self.smtp.login(self.mailuser, self.mailpass)
            self.loggedin = True

        emailMsg['From']=self.fromaddr
        self.smtp.sendmail(emailMsg['From'], emailMsg['To'], emailMsg.as_string())

    def __exit__(self):
        print("exiting MailBackend")
        self.smtp.quit()




"""
CronWrapper
~~~~~~~~~~~~~~
Main module linking together mail backend, helper scripts for, e.g., output trimming;
handles command lines as parsed by ccscript

    :copyright (cronwrap): 2010 by Plurk
    :copyright: 2015 by Matthias Kauer
    :license: BSD
"""

import platform
import sys
import email
from croncoat.cc.expiringcommand import ExpiringCommand
from croncoat.cc.mailbackend import MailBackend
from croncoat.cc.helper import Helper

class CronWrapper(object):
    def __init__(self, sys_args, scriptname, configpath):
        self.scriptname = scriptname
        self.subjectname = "cc"

        if not sys_args.time:
            sys_args.time = '1h'

        if sys_args.verbose is not False:
            sys_args.verbose = True

        self.sys_args = sys_args
        self.mailer = MailBackend(configpath)

    def run(self):
        sys_args = self.sys_args
        if sys_args.cmd:
            self.cmd = ExpiringCommand( sys_args.cmd, sys_args.time)
            self.cmd.Run()
            if self.cmd.returncode != 0:
                self.handle_error()
            elif Helper.is_time_exceeded(self.sys_args, self.cmd):
                self.handle_timeout()
            else:
                self.handle_success()
        elif sys_args.emails:
            self.handle_test_email()

    @staticmethod
    def _ini_string(scriptname):
        return (
"""#example format for ~/.{}.ini (don't use quotes!)
[Mail]
smtpserver=
smtpport=
user=
pass=
fromaddr=
security=ssl    ;ssl, starttls or none
""".format(scriptname))

    @staticmethod
    def print_ini(scriptname):
        print CronWrapper._ini_string(scriptname)

    def handle_success(self):
        sys_args = self.sys_args; cmd = self.cmd
        """Called if a command did finish successfuly."""
        content_elem = 'RAN COMMAND SUCCESSFULLY'
        subj_elem='success'

        if sys_args.verbose:
            self.handle_general(subj_elem=subj_elem, content_elem=content_elem);

    def handle_general(self, subj_elem, content_elem):
        sys_args = self.sys_args; cmd = self.cmd
        out_str = Helper.render_email_template('%s %s: ' % \
                (self.scriptname, content_elem), sys_args, cmd)
        subj_str='%s (%s): %s [%s]' % \
                (sys_args.cmd[:20],
                            platform.node().capitalize(), subj_elem, self.scriptname)

        if sys_args.emails:
            self.send_email(subject=subj_str, content=out_str)
            #  self.handle_general(subj_str=subj_str, content_str=out_str);
        else:
            print out_str

    def handle_timeout(self):
        """Called if a command exceeds its running time."""

        sys_args = self.sysargs; cmd = self.cmd
        content_elem = 'DETECTED A TIMEOUT ON FOLLOWING COMMAND'
        subj_elem= 'timeout'

        self.handle_general(subj_elem=subj_elem, content_elem=content_elem);

    def handle_error(self):
        """Called when a command did not finish successfully."""
        sys_args = self.sys_args; cmd = self.cmd

        content_elem = 'DETECTED FAILURE OR ERROR OUTPUT FOR THE COMMAND'
        subj_elem= 'failure'

        self.handle_general(subj_elem=subj_elem, content_elem=content_elem);
        sys.exit(-1)

    def handle_test_email(self):
        subject='%s (%s): Testing' % \
                (self.subjectname, platform.node().capitalize())
        content = 'just a test mail, yo! :)'
        self.send_email(subject, content)


    def send_email(self, subject, content):
        """Sends an email via MailBackend (python email lib there)."""
        emails = self.sys_args.emails
        emails = emails.split(',')

        for toaddr in emails:
            emailMsg = email.MIMEMultipart.MIMEMultipart('mixed')
            emailMsg['To'] = toaddr
            #             emailMsg['From'] = sys_args.fromaddr
            emailMsg['Subject'] = subject.replace('"', "'")
            emailMsg.attach(email.mime.Text.MIMEText(content, _charset='utf-8'))
            self.mailer.sendmail(emailMsg)




"""
ExpiringCommand
===============
Runs a command but terminates it when the timeout expires.

This is a minimal wrapper around Popen with timeout from subprocess32
(backport of Py3 functionality).

    :copyright: 2017 Matthias Kauer
    :license: BSD
"""

import time
import subprocess32 as sp
import shlex
from croncoat.cc.helper import Helper


class ExpiringCommand(object):
    def __init__(self, command, timeout):
        self.cmd = shlex.split(command)
        self.timeout = Helper.parse_time_to_secs(timeout)
        self.start_time = time.time()

        self.stdout = "No stdout captured (yet?)"
        self.stderr = "No stderr captured (yet?)"

    def Run(self):
        self.p = sp.Popen(self.cmd,stderr=sp.PIPE, stdout=sp.PIPE, shell=False,
                universal_newlines=True)

        try:
            self.stdout, self.stderr = self.p.communicate(timeout=self.timeout)
            self.returncode = self.p.returncode
        except sp.TimeoutExpired:
            self.p.terminate()
            self.stdout, self.stderr = self.p.communicate()
            self.returncode = 1
        finally:
            self.run_time = time.time() - self.start_time




#!/usr/bin/env python

import os
from setuptools import setup, find_packages
from croncoat import __version__

#following PyPI guide: https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/

def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()

setup(name='croncoat',
    version = __version__,
    author="Matthias Kauer",
    author_email="mk.software@zuez.org",
    url="https://github.com/MatthiasKauer/croncoat",
    classifiers=[
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: BSD License",
    "Operating System :: Unix",
    "Programming Language :: Python",
    ],

    install_requires=[
      'argparse>=1.1',
      'subprocess32>=3.2.7',
    ],
    packages = find_packages(),

    entry_points={'console_scripts': ['croncoat=croncoat.scripts.ccscript:main']},
    platforms=["Unix"],
    license="BSD",
    keywords='cron wrapper crontab cronjob email',
    description="croncoat extends cronwrap, a cron job wrapper that wraps jobs and enables better error reporting and command timeouts.",
    long_description=read('README.md'),
)
